version = "2.32.1"

user = None
language = "en-US"
timeout = 15
request_methods_prefix = "Joker"

from starkcore import Organization, Project, MarketplaceApp

from . import error
from . import key

from . import transaction
from .transaction.__transaction import Transaction

from . import balance
from .balance.__balance import Balance

from . import boleto
from .boleto.__boleto import Boleto

from . import transfer
from .transfer.__transfer import Transfer

from . import institution
from .institution.__institution import Institution

from . import boletoholmes
from .boletoholmes.__boletoholmes import BoletoHolmes

from . import brcodepayment
from .brcodepayment.__brcodepayment import BrcodePayment

from . import boletopayment
from .boletopayment.__boletopayment import BoletoPayment

from . import corporatetransaction
from .corporatetransaction.__corporatetransaction import CorporateTransaction

from . import corporateholder
from .corporateholder.__corporateholder import CorporateHolder

from . import corporatebalance
from .corporatebalance.__corporatebalance import CorporateBalance

from . import corporatecard
from .corporatecard.__corporatecard import CorporateCard

from . import corporatepurchase
from .corporatepurchase.__corporatepurchase import CorporatePurchase

from . import corporateinvoice
from .corporateinvoice.__corporateinvoice import CorporateInvoice

from . import corporatewithdrawal
from .corporatewithdrawal.__corporatewithdrawal import CorporateWithdrawal

from . import corporaterule
from .corporaterule.__corporaterule import CorporateRule

from . import merchantcard
from .merchantcard.__merchantcard import MerchantCard

from . import merchantcategory
from .merchantcategory.__merchantcategory import MerchantCategory

from . import merchantcountry
from .merchantcountry.__merchantcountry import MerchantCountry

from . import merchantinstallment
from .merchantinstallment.__merchantinstallment import MerchantInstallment

from . import merchantpurchase
from .merchantpurchase.__merchantpurchase import MerchantPurchase

from . import merchantsession
from .merchantsession.__merchantsession import MerchantSession

from . import cardmethod
from .cardmethod.__cardmethod import CardMethod

from . import utilitypayment
from .utilitypayment.__utilitypayment import UtilityPayment

from . import taxpayment
from .taxpayment.__taxpayment import TaxPayment

from . import darfpayment
from .darfpayment.__darfpayment import DarfPayment

from . import webhook
from .webhook.__webhook import Webhook

from . import workspace
from .workspace.__workspace import Workspace

from . import event
from .event.__event import Event

from . import paymentpreview
from .paymentpreview.__paymentpreview import PaymentPreview

from . import paymentrequest
from .paymentrequest.__paymentrequest import PaymentRequest

from . import invoice
from .invoice.__invoice import Invoice

from . import invoicepullrequest
from .invoicepullrequest.__invoicepullrequest import InvoicePullRequest

from . import invoicepullsubscription
from .invoicepullsubscription.__invoicepullsubscription import InvoicePullSubscription

from . import dictkey
from .dictkey.__dictkey import DictKey

from . import dynamicbrcode
from .dynamicbrcode.__dynamicbrcode import DynamicBrcode

from . import deposit
from .deposit.__deposit import Deposit

from . import split
from .split.__split import Split

from . import splitreceiver
from .splitreceiver.__splitreceiver import SplitReceiver

from . import splitprofile
from .splitprofile.__splitprofile import SplitProfile

from . import request




from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date
from ..utils import rest


class CorporateWithdrawal(Resource):
    """# CorporateWithdrawal object
    The CorporateWithdrawal objects created in your Workspace return cash from your Corporate balance to your
    Banking balance.
    ## Parameters (required):
    - amount [integer]: CorporateWithdrawal value in cents. Minimum = 0 (any value will be accepted). ex: 1234 (= R$ 12.34)
    - external_id [string] CorporateWithdrawal external ID. ex: "12345"
    ## Parameters (optional):
    - tags [list of strings, default []]: list of strings for tagging. ex: ["tony", "stark"]
    ## Attributes (return-only):
    - id [string]: unique id returned when CorporateWithdrawal is created. ex: "5656565656565656"
    - transaction_id [string]: Stark Bank ledger transaction ids linked to this CorporateWithdrawal
    - corporate_transaction_id [string]: corporate ledger transaction ids linked to this CorporateWithdrawal
    - updated [datetime.datetime]: latest update datetime for the CorporateWithdrawal. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - created [datetime.datetime]: creation datetime for the CorporateWithdrawal. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, amount, external_id, tags=None, id=None, transaction_id=None,
                 corporate_transaction_id=None, updated=None, created=None):
        Resource.__init__(self, id=id)

        self.amount = amount
        self.external_id = external_id
        self.tags = tags
        self.transaction_id = transaction_id
        self.corporate_transaction_id = corporate_transaction_id
        self.updated = check_datetime(updated)
        self.created = check_datetime(created)


_resource = {"class": CorporateWithdrawal, "name": "CorporateWithdrawal"}


def create(withdrawal, user=None):
    """# Create a CorporateWithdrawal
    Send a single CorporateWithdrawal object for creation at the Stark Bank API
    ## Parameters (required):
    - withdrawal [CorporateWithdrawal object]: CorporateWithdrawal object to be created in the API.
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call.
    ## Return:
    - CorporateWithdrawal object with updated attributes
    """
    return rest.post_single(resource=_resource, entity=withdrawal, user=user)


def get(id, user=None):
    """# Retrieve a specific CorporateWithdrawal
    Receive a single CorporateWithdrawal object previously created in the Stark Bank API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call.
    ## Return:
    - CorporateWithdrawal object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def query(external_ids=None, after=None, before=None, limit=None, tags=None, user=None):
    """# Retrieve CorporateWithdrawals
    Receive a generator of CorporateWithdrawal objects previously created in the Stark Bank API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - external_ids [list of strings, default None]: external IDs. ex: ["5656565656565656", "4545454545454545"]
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call.
    ## Return:
    - generator of CorporateWithdrawal objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        external_ids=external_ids,
        after=check_date(after),
        before=check_date(before),
        tags=tags,
        limit=limit,
        user=user,
    )


def page(external_ids=None, after=None, before=None, limit=None, tags=None, cursor=None, user=None):
    """# Retrieve paged CorporateWithdrawals
    Receive a list of up to 100 CorporateWithdrawal objects previously created in the Stark Bank API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. Max = 100. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - external_ids [list of strings, default None]: external IDs. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call.
    ## Return:
    - list of CorporateWithdrawal objects with updated attributes
    - cursor to retrieve the next page of CorporateWithdrawal objects
    """
    return rest.get_page(
        resource=_resource,
        external_ids=external_ids,
        after=check_date(after),
        before=check_date(before),
        tags=tags,
        limit=limit,
        cursor=cursor,
        user=user,
    )



from starkcore.utils.api import from_api_json
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date
from ...utils import rest
from ..__corporatecard import _resource as _corporate_card_resource


class Log(Resource):
    """# corporatecard.Log object
    Every time a CorporateCard entity is updated, a corresponding corporatecard.Log
    is generated for the entity. This log is never generated by the
    user, but it can be retrieved to check additional information
    on the CorporateCard.
    ## Attributes (return-only):
    - id [string]: unique id returned when the log is created. ex: "5656565656565656"
    - card [CorporateCard]: CorporateCard entity to which the log refers to.
    - type [string]: type of the CorporateCard event which triggered the log creation. ex: "blocked", "canceled", "created", "expired", "unblocked", "updated"
    - created [datetime.datetime]: creation datetime for the log. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, id, card, type, created):
        Resource.__init__(self, id=id)

        self.card = from_api_json(_corporate_card_resource, card)
        self.type = type
        self.created = check_datetime(created)


_resource = {"class": Log, "name": "CorporateCardLog"}


def get(id, user=None):
    """# Retrieve a specific corporatecard.Log
    Receive a single corporatecard.Log object previously created by the Stark Bank API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call.
    ## Return:
    - corporatecard.Log object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def query(limit=None, card_ids=None, types=None, after=None, before=None, ids=None, user=None):
    """# Retrieve corporatecard.Log
    Receive a generator of corporatecard.Log objects previously created in the Stark Bank API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - types [list of strings, default None]: filter for log event types. ex: ["blocked", "canceled", "created", "expired", "unblocked", "updated"]
    - card_ids [list of strings, default None]: list of CorporateCard ids to filter logs. ex: ["5656565656565656", "4545454545454545"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call.
    ## Return:
    - generator of corporatecard.Log objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        ids=ids,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        types=types,
        card_ids=card_ids,
        user=user,
    )


def page(cursor=None, limit=None, after=None, before=None, types=None, card_ids=None, ids=None, user=None):
    """# Retrieve paged corporatecard.Log
    Receive a list of up to 100 corporatecard.Log objects previously created in the Stark Bank API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. It must be an integer between 1 and 100. ex: 50
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - types [list of strings, default None]: filter for log event types. ex: ["blocked", "canceled", "created", "expired", "unblocked", "updated"]
    - card_ids [list of strings, default None]: list of CorporateCard ids to filter logs. ex: ["5656565656565656", "4545454545454545"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call.
    ## Return:
    - list of corporatecard.Log objects with updated attributes
    - cursor to retrieve the next page of corporatecard.Log objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        types=types,
        card_ids=card_ids,
        ids=ids,
        user=user,
    )



from .__paymentpreview import create
from .__brcodepreview import BrcodePreview
from .__boletopreview import BoletoPreview
from .__utilitypreview import UtilityPreview
from .__taxpreview import TaxPreview



from ...utils import rest
from starkcore.utils.api import from_api_json
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date
from ..__deposit import _resource as _deposit_resource


class Log(Resource):
    """# deposit.Log object
    Every time a Deposit entity is updated, a corresponding deposit.Log
    is generated for the entity. This log is never generated by the
    user, but it can be retrieved to check additional information
    on the Deposit.
    ## Attributes (return-only):
    - id [string]: unique id returned when the log is created. ex: "5656565656565656"
    - deposit [Deposit]: Deposit entity to which the log refers to.
    - errors [list of strings]: list of errors linked to this Deposit event
    - type [string]: type of the Deposit event which triggered the log creation. ex: "created" or "credited"
    - created [datetime.datetime]: creation datetime for the log. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, id, created, type, errors, deposit):
        Resource.__init__(self, id=id)

        self.created = check_datetime(created)
        self.type = type
        self.errors = errors
        self.deposit = from_api_json(_deposit_resource, deposit)


_resource = {"class": Log, "name": "DepositLog"}


def get(id, user=None):
    """# Retrieve a specific deposit.Log
    Receive a single deposit.Log object previously created by the Stark Bank API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - deposit.Log object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def query(limit=None, after=None, before=None, types=None, deposit_ids=None, user=None):
    """# Retrieve deposit.Logs
    Receive a generator of deposit.Log objects previously created in the Stark Bank API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - types [list of strings, default None]: filter for log event types. ex: "created" or "credited"
    - deposit_ids [list of strings, default None]: list of Deposit ids to filter logs. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - generator of deposit.Log objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        types=types,
        deposit_ids=deposit_ids,
        user=user,
    )


def page(cursor=None, limit=None, after=None, before=None, types=None, deposit_ids=None, user=None):
    """# Retrieve paged deposit.Logs
    Receive a list of up to 100 deposit.Log objects previously created in the Stark Bank API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. It must be an integer between 1 and 100. ex: 50
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - types [list of strings, default None]: filter for log event types. ex: "created" or "credited"
    - deposit_ids [list of strings, default None]: list of Deposit ids to filter logs. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of deposit.Log objects with updated attributes
    - cursor to retrieve the next page of deposit.Log objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        types=types,
        deposit_ids=deposit_ids,
        user=user,
    )

def pdf(id, user=None):
    """# Retrieve a reversed Deposit.Log pdf file
    Receive a reversed Deposit.Log pdf receipt file generated in the Stark Bank API by its id.
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - Deposit pdf file
    """
    return rest.get_content(resource=_resource, id=id, user=user, sub_resource_name="pdf")



from ..utils import rest
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date


class BoletoPayment(Resource):
    """# BoletoPayment object
    When you initialize a BoletoPayment, the entity will not be automatically
    created in the Stark Bank API. The 'create' function sends the objects
    to the Stark Bank API and returns the list of created objects.
    ## Parameters (conditionally required):
    - line [string, default None]: Number sequence that describes the payment. Either 'line' or 'bar_code' parameters are required. If both are sent, they must match. ex: "34191.09008 63571.277308 71444.640008 5 81960000000062"
    - bar_code [string, default None]: Bar code number that describes the payment. Either 'line' or 'barCode' parameters are required. If both are sent, they must match. ex: "34195819600000000621090063571277307144464000"
    ## Parameters (required):
    - tax_id [string]: receiver tax ID (CPF or CNPJ) with or without formatting. ex: "01234567890" or "20.018.183/0001-80"
    - description [string]: Text to be displayed in your statement (min. 10 characters). ex: "payment ABC"
    ## Parameters (optional):
    - amount [int, default None]: amount to be paid. If none is informed, the current boleto value will be used. ex: 23456 (= R$ 234.56)
    - scheduled [datetime.date or string, default today]: payment scheduled date. ex: datetime.date(2020, 3, 10)
    - tags [list of strings]: list of strings for tagging
    ## Attributes (return-only):
    - id [string]: unique id returned when payment is created. ex: "5656565656565656"
    - status [string]: current payment status. ex: "success" or "failed"
    - fee [integer]: fee charged when the boleto payment is created. ex: 200 (= R$ 2.00)
    - transaction_ids [list of strings]: ledger transaction ids linked to this BoletoPayment. ex: ["19827356981273"]
    - created [datetime.datetime]: creation datetime for the payment. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, tax_id, description, line=None, bar_code=None, scheduled=None, tags=None, id=None, status=None, amount=None, fee=None, transaction_ids=None, created=None):
        Resource.__init__(self, id=id)

        self.line = line
        self.tax_id = tax_id
        self.bar_code = bar_code
        self.description = description
        self.tags = tags
        self.scheduled = check_date(scheduled)
        self.status = status
        self.amount = amount
        self.fee = fee
        self.transaction_ids = transaction_ids
        self.created = check_datetime(created)


_resource = {"class": BoletoPayment, "name": "BoletoPayment"}


def create(payments, user=None):
    """# Create BoletoPayments
    Send a list of BoletoPayment objects for creation in the Stark Bank API
    ## Parameters (required):
    - payments [list of BoletoPayment objects]: list of BoletoPayment objects to be created in the API
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of BoletoPayment objects with updated attributes
    """
    return rest.post_multi(resource=_resource, entities=payments, user=user)


def get(id, user=None):
    """# Retrieve a specific BoletoPayment
    Receive a single BoletoPayment object previously created by the Stark Bank API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - BoletoPayment object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def pdf(id, user=None):
    """# Retrieve a specific BoletoPayment pdf file
    Receive a single BoletoPayment pdf file generated in the Stark Bank API by its id.
    Only valid for boleto payments with "success" status.
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - BoletoPayment pdf file
    """
    return rest.get_content(resource=_resource, id=id, user=user, sub_resource_name="pdf")


def query(limit=None, after=None, before=None, tags=None, ids=None, status=None, user=None):
    """# Retrieve BoletoPayments
    Receive a generator of BoletoPayment objects previously created in the Stark Bank API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - status [string, default None]: filter for status of retrieved objects. ex: "success"
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - generator of BoletoPayment objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        tags=tags,
        ids=ids,
        status=status,
        user=user,
    )


def page(cursor=None, limit=None, after=None, before=None, tags=None, ids=None, status=None, user=None):
    """# Retrieve paged BoletoPayments
    Receive a list of up to 100 BoletoPayment objects previously created in the Stark Bank API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. It must be an integer between 1 and 100. ex: 50
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - status [string, default None]: filter for status of retrieved objects. ex: "success"
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of BoletoPayment objects with updated attributes
    - cursor to retrieve the next page of BoletoPayment objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        tags=tags,
        ids=ids,
        status=status,
        user=user,
    )


def delete(id, user=None):
    """# Delete a BoletoPayment entity
    Delete a BoletoPayment entity previously created in the Stark Bank API
    ## Parameters (required):
    - id [string]: BoletoPayment unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - deleted BoletoPayment object
    """
    return rest.delete_id(resource=_resource, id=id, user=user)



from ..utils import rest
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_date, check_datetime


class Deposit(Resource):

    """# Deposit object
    Deposits represent passive cash-in received by your account from external transfers
    ## Attributes (return-only):
    - id [string]: unique id associated with a Deposit when it is created. ex: "5656565656565656"
    - name [string]: payer name. ex: "Iron Bank S.A."
    - tax_id [string]: payer tax ID (CPF or CNPJ). ex: "012.345.678-90" or "20.018.183/0001-80"
    - bank_code [string]: payer bank code in Brazil. ex: "20018183" or "341"
    - branch_code [string]: payer bank account branch. ex: "1357-9"
    - account_number [string]: payer bank account number. ex: "876543-2"
    - account_type [string]: payer bank account type. ex: "checking"
    - amount [integer]: Deposit value in cents. ex: 1234 (= R$ 12.34)
    - type [string]: Type of settlement that originated the deposit. ex: "pix" or "ted"
    - status [string]: current Deposit status. ex: "created"
    - tags [list of strings]: list of strings that are tagging the deposit. ex: ["reconciliationId", "txId"]
    - fee [integer]: fee charged by this deposit. ex: 50 (= R$ 0.50)
    - transaction_ids [list of strings]: ledger transaction ids linked to this deposit (if there are more than one, all but the first are reversals or failed reversal chargebacks). ex: ["19827356981273"]
    - created [datetime.datetime]: creation datetime for the Deposit. ex: datetime.datetime(2020, 12, 10, 10, 30, 0, 0)
    - updated [datetime.datetime]: latest update datetime for the Deposit. ex: datetime.datetime(2020, 12, 10, 10, 30, 0, 0)
    """

    def __init__(self, id, name, tax_id, bank_code, branch_code, account_number, account_type, amount, type, status,
                 tags, fee, transaction_ids, created, updated):
        Resource.__init__(self, id=id)

        self.name = name
        self.tax_id = tax_id
        self.bank_code = bank_code
        self.branch_code = branch_code
        self.account_number = account_number
        self.account_type = account_type
        self.amount = amount
        self.type = type
        self.status = status
        self.tags = tags
        self.fee = fee
        self.transaction_ids = transaction_ids
        self.created = check_datetime(created)
        self.updated = check_datetime(updated)


_resource = {"class": Deposit, "name": "Deposit"}


def get(id, user=None):
    """# Retrieve a specific Deposit
    Receive a single Deposit object from the Stark Bank API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - Deposit object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def query(limit=None, after=None, before=None, status=None, sort=None, tags=None, ids=None, user=None):
    """# Retrieve Deposits
    Receive a generator of Deposit objects from the Stark Bank API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - status [string, default None]: filter for status of retrieved objects. ex: "paid" or "registered"
    - sort [string, default "-created"]: sort order considered in response. Valid options are "created" or "-created".
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - generator of Deposit objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        sort=sort,
        status=status,
        tags=tags,
        ids=ids,
        user=user,
    )


def page(cursor=None, limit=None, after=None, before=None, status=None, sort=None, tags=None, ids=None, user=None):
    """# Retrieve paged Deposits
    Receive a list of up to 100 Deposit objects previously created in the Stark Bank API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. It must be an integer between 1 and 100. ex: 50
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - status [string, default None]: filter for status of retrieved objects. ex: "paid" or "registered"
    - sort [string, default "-created"]: sort order considered in response. Valid options are "created" or "-created".
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of Deposit objects with updated attributes
    - cursor to retrieve the next page of Deposit objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        sort=sort,
        status=status,
        tags=tags,
        ids=ids,
        user=user,
    )


def update(id, amount=None, user=None):
    """# Update Deposit entity
    Update the Deposit by passing its id to be partially or fully reversed.
    ## Parameters (required):
    - id [string]: Deposit id. ex: "5656565656565656"
    ## Parameters (optional):
    - amount [string, default None]: The new amount of the Deposit. If the amount = 0 the Deposit will be fully reversed
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - target Deposit with updated attributes
    """
    payload = {
        "amount": amount,
    }
    return rest.patch_id(resource=_resource, id=id, user=user, payload=payload)


from ..utils import rest
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_date, check_datetime


class Boleto(Resource):
    """# Boleto object
    When you initialize a Boleto, the entity will not be automatically
    sent to the Stark Bank API. The 'create' function sends the objects
    to the Stark Bank API and returns the list of created objects.
    ## Parameters (required):
    - amount [integer]: Boleto value in cents. Minimum = 200 (R$2,00). ex: 1234 (= R$ 12.34)
    - name [string]: payer full name. ex: "Anthony Edward Stark"
    - tax_id [string]: payer tax ID (CPF or CNPJ) with or without formatting. ex: "01234567890" or "20.018.183/0001-80"
    - street_line_1 [string]: payer main address. ex: Av. Paulista, 200
    - street_line_2 [string]: payer address complement. ex: Apto. 123
    - district [string]: payer address district / neighbourhood. ex: Bela Vista
    - city [string]: payer address city. ex: Rio de Janeiro
    - state_code [string]: payer address state. ex: GO
    - zip_code [string]: payer address zip code. ex: 01311-200
    ## Parameters (optional):
    - due [datetime.date or string, default today + 2 days]: Boleto due date in ISO format. ex: 2020-04-30
    - fine [float, default 2.0]: Boleto fine for overdue payment in %. ex: 2.5
    - interest [float, default 1.0]: Boleto monthly interest for overdue payment in %. ex: 5.2
    - overdue_limit [integer, default 59]: limit in days for payment after due date. ex: 7 (max: 59)
    - descriptions [list of dictionaries, default None]: list of dictionaries with "text":string and (optional) "amount":int pairs
    - discounts [list of dictionaries, default None]: list of dictionaries with "percentage":float and "date":datetime.datetime or string pairs
    - tags [list of strings]: list of strings for tagging
    - receiver_name [string]: receiver (Sacador Avalista) full name. ex: "Anthony Edward Stark"
    - receiver_tax_id [string]: receiver (Sacador Avalista) tax ID (CPF or CNPJ) with or without formatting. ex: "01234567890" or "20.018.183/0001-80"
    ## Attributes (return-only):
    - id [string]: unique id returned when Boleto is created. ex: "5656565656565656"
    - fee [integer]: fee charged when Boleto is paid. ex: 200 (= R$ 2.00)
    - line [string]: generated Boleto line for payment. ex: "34191.09008 63571.277308 71444.640008 5 81960000000062"
    - bar_code [string]: generated Boleto bar-code for payment. ex: "34195819600000000621090063571277307144464000"
    - status [string]: current Boleto status. ex: "registered" or "paid"
    - transaction_ids [list of strings]: ledger transaction ids linked to this boleto. ex: ["19827356981273"]
    - workspace_id [string]: ID of the Workspace where this Boleto was generated. ex: "4545454545454545"
    - our_number [string]: Reference number registered at the settlement bank. ex:"10131474"
    - created [datetime.datetime]: creation datetime for the Boleto. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, amount, name, tax_id, street_line_1, street_line_2, district, city, state_code, zip_code,
                 due=None, fine=None, interest=None, overdue_limit=None, tags=None, descriptions=None, discounts=None,
                 receiver_name=None, receiver_tax_id=None, id=None, fee=None, line=None, bar_code=None, status=None,
                 workspace_id=None, transaction_ids=None, created=None, our_number=None):
        Resource.__init__(self, id=id)

        self.amount = amount
        self.fee = fee
        self.name = name
        self.tax_id = tax_id
        self.street_line_1 = street_line_1
        self.street_line_2 = street_line_2
        self.district = district
        self.city = city
        self.state_code = state_code
        self.zip_code = zip_code
        self.receiver_name = receiver_name
        self.receiver_tax_id = receiver_tax_id
        self.due = check_date(due)
        self.fine = fine
        self.interest = interest
        self.overdue_limit = overdue_limit
        self.tags = tags
        self.descriptions = descriptions
        self.discounts = discounts
        self.line = line
        self.bar_code = bar_code
        self.status = status
        self.transaction_ids = transaction_ids
        self.workspace_id = workspace_id
        self.created = check_datetime(created)
        self.our_number = our_number


_resource = {"class": Boleto, "name": "Boleto"}


def create(boletos, user=None):
    """# Create Boletos
    Send a list of Boleto objects for creation in the Stark Bank API
    ## Parameters (required):
    - boletos [list of Boleto objects]: list of Boleto objects to be created in the API
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of Boleto objects with updated attributes
    """
    return rest.post_multi(resource=_resource, entities=boletos, user=user)


def get(id, user=None):
    """# Retrieve a specific Boleto
    Receive a single Boleto object previously created in the Stark Bank API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - Boleto object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def pdf(id, layout=None, hidden_fields=None, user=None):
    """# Retrieve a specific Boleto pdf file
    Receive a single Boleto pdf file generated in the Stark Bank API by its id.
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - layout [string]: Layout specification. Available options are "default" and "booklet"
    - hidden_fields [list of strings, default None]: List of string fields to be hidden in Boleto pdf. ex: ["customerAddress"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - Boleto pdf file
    """
    return rest.get_content(resource=_resource, id=id, layout=layout, hidden_fields=hidden_fields, user=user, sub_resource_name="pdf")


def query(limit=None, status=None, tags=None, ids=None, after=None, before=None, user=None):
    """# Retrieve Boletos
    Receive a generator of Boleto objects previously created in the Stark Bank API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - status [string, default None]: filter for status of retrieved objects. ex: "paid" or "registered"
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - generator of Boleto objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        tags=tags,
        ids=ids,
        user=user,
    )


def page(cursor=None, limit=None, status=None, tags=None, ids=None, after=None, before=None, user=None):
    """# Retrieve paged Boletos
    Receive a list of up to 100 Boleto objects previously created in the Stark Bank API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. It must be an integer between 1 and 100. ex: 50
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - status [string, default None]: filter for status of retrieved objects. ex: "paid" or "registered"
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of Boleto objects with updated attributes
    - cursor to retrieve the next page of Boleto objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        tags=tags,
        ids=ids,
        user=user,
    )


def delete(id, user=None):
    """# Delete a Boleto entity
    Delete a Boleto entity previously created in the Stark Bank API
    ## Parameters (required):
    - id [string]: Boleto unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - deleted Boleto object
    """
    return rest.delete_id(resource=_resource, id=id, user=user)



from ..utils import rest
from starkcore.utils.resource import SubResource


class Institution(SubResource):
    """# Institution object
    This resource is used to get information on the institutions that are recognized by the Brazilian Central Bank.
    Besides the display name and full name, they also include the STR code (used for TEDs) and the SPI Code
    (used for Pix) for the institutions. Either of these codes may be empty if the institution is not registered on
    that Central Bank service.
    ## Attributes (return-only):
    - display_name [string]: short version of the institution name that should be displayed to end users. ex: "Stark Bank"
    - name [string]: full version of the institution name. ex: "Stark Bank S.A."
    - spi_code [string]: SPI code used to identify the institution on Pix transactions. ex: "20018183"
    - str_code [string]: STR code used to identify the institution on TED transactions. ex: "123"
    """

    def __init__(self, display_name, name, spi_code, str_code):
        self.display_name = display_name
        self.name = name
        self.spi_code = spi_code
        self.str_code = str_code


_resource = {"class": Institution, "name": "Institution"}


def query(limit=None, search=None, spi_codes=None, str_codes=None, user=None):
    """# Retrieve Bacen Institutions
    Receive a list of Institution objects that are recognized by the Brazilian Central bank for Pix and TED transactions
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - search [string, default None]: part of the institution name to be searched. ex: "stark"
    - spi_codes [list of strings, default None]: list of SPI (Pix) codes to be searched. ex: ["20018183"]
    - str_codes [list of strings, default None]: list of STR (TED) codes to be searched. ex: ["260"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of Institution objects with updated attributes
    """
    return rest.get_page(resource=_resource, search=search, spi_codes=spi_codes, str_codes=str_codes, limit=limit, user=user)[0]



from ...utils import rest
from starkcore.utils.resource import Resource
from starkcore.utils.api import from_api_json
from starkcore.utils.checks import check_date, check_datetime
from ..__merchantpurchase import _resource as _merchant_purchase_resource


class Log(Resource):
    """# merchantpurchase.Log object
    Check out our API Documentation at https://starkbank.com/docs/api#merchant-purchase
    """

    def __init__(self, id, created, type, errors, purchase):
        Resource.__init__(self, id=id)

        self.created = check_datetime(created)
        self.type = type
        self.errors = errors
        self.purchase = from_api_json(_merchant_purchase_resource, purchase)


_resource = {"class": Log, "name": "MerchantPurchaseLog"}


def get(id, user=None):
    return rest.get_id(resource=_resource, id=id, user=user)


def query(limit=None, after=None, before=None, types=None, user=None, purchase_ids=None):
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        types=types,
        user=user,
        purchase_ids=purchase_ids,
    )

def page(cursor=None, limit=None, after=None, before=None, types=None, user=None, purchase_ids=None):
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        types=types,
        user=user,
        purchase_ids=purchase_ids,
    )




from ..utils import rest
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date


class UtilityPayment(Resource):
    """# UtilityPayment object
    When you initialize a UtilityPayment, the entity will not be automatically
    created in the Stark Bank API. The 'create' function sends the objects
    to the Stark Bank API and returns the list of created objects.
    ## Parameters (conditionally required):
    - line [string, default None]: Number sequence that describes the payment. Either 'line' or 'bar_code' parameters are required. If both are sent, they must match. ex: "34191.09008 63571.277308 71444.640008 5 81960000000062"
    - bar_code [string, default None]: Bar code number that describes the payment. Either 'line' or 'barCode' parameters are required. If both are sent, they must match. ex: "34195819600000000621090063571277307144464000"
    ## Parameters (required):
    - description [string]: Text to be displayed in your statement (min. 10 characters). ex: "payment ABC"
    ## Parameters (optional):
    - scheduled [datetime.date or string, default today]: payment scheduled date. ex: datetime.date(2020, 3, 10)
    - tags [list of strings]: list of strings for tagging
    ## Attributes (return-only):
    - id [string]: unique id returned when payment is created. ex: "5656565656565656"
    - status [string]: current payment status. ex: "success" or "failed"
    - amount [int]: amount automatically calculated from line or bar_code. ex: 23456 (= R$ 234.56)
    - fee [integer]: fee charged when utility payment is created. ex: 200 (= R$ 2.00)
    - type [string]: payment type. ex: "utility"
    - transaction_ids [list of strings]: ledger transaction ids linked to this UtilityPayment. ex: ["19827356981273"]
    - created [datetime.datetime]: creation datetime for the payment. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - updated [datetime.datetime]: latest update datetime for the payment. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, description, line=None, bar_code=None, tags=None, scheduled=None, id=None, type=None,
                 amount=None, fee=None, transaction_ids=None, status=None, created=None, updated=None):
        Resource.__init__(self, id=id)

        self.line = line
        self.bar_code = bar_code
        self.description = description
        self.tags = tags
        self.scheduled = check_date(scheduled)
        self.status = status
        self.amount = amount
        self.fee = fee
        self.type = type
        self.transaction_ids = transaction_ids
        self.created = check_datetime(created)
        self.updated = check_datetime(updated)


_resource = {"class": UtilityPayment, "name": "UtilityPayment"}


def create(payments, user=None):
    """# Create UtilityPayments
    Send a list of UtilityPayment objects for creation in the Stark Bank API
    ## Parameters (required):
    - payments [list of UtilityPayment objects]: list of UtilityPayment objects to be created in the API
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of UtilityPayment objects with updated attributes
    """
    return rest.post_multi(resource=_resource, entities=payments, user=user)


def get(id, user=None):
    """# Retrieve a specific UtilityPayment
    Receive a single UtilityPayment object previously created by the Stark Bank API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - UtilityPayment object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def pdf(id, user=None):
    """# Retrieve a specific UtilityPayment pdf file
    Receive a single UtilityPayment pdf file generated in the Stark Bank API by its id.
    Only valid for utility payments with "success" status.
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - UtilityPayment pdf file
    """
    return rest.get_content(resource=_resource, id=id, user=user, sub_resource_name="pdf")


def query(limit=None, after=None, before=None, tags=None, ids=None, status=None, user=None):
    """# Retrieve UtilityPayments
    Receive a generator of UtilityPayment objects previously created in the Stark Bank API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - status [string, default None]: filter for status of retrieved objects. ex: "success"
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - generator of UtilityPayment objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        tags=tags,
        ids=ids,
        status=status,
        user=user,
    )


def page(cursor=None, limit=None, after=None, before=None, tags=None, ids=None, status=None, user=None):
    """# Retrieve paged UtilityPayments
    Receive a list of up to 100 UtilityPayment objects previously created in the Stark Bank API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. It must be an integer between 1 and 100. ex: 50
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - status [string, default None]: filter for status of retrieved objects. ex: "success"
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of UtilityPayment objects with updated attributes
    - cursor to retrieve the next page of UtilityPayment objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        tags=tags,
        ids=ids,
        status=status,
        user=user,
    )


def delete(id, user=None):
    """# Delete a UtilityPayment entity
    Delete a UtilityPayment entity previously created in the Stark Bank API
    ## Parameters (required):
    - id [string]: UtilityPayment unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - deleted UtilityPayment object
    """
    return rest.delete_id(resource=_resource, id=id, user=user)



from ..utils import rest
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date
from starkbank.error import StarkError


class Transaction(Resource):
    """# Transaction object
    A Transaction is a transfer of funds between workspaces inside Stark Bank.
    Transactions created by the user are only for internal transactions.
    Other operations (such as transfer or charge-payment) will automatically
    create a transaction for the user which can be retrieved for the statement.
    When you initialize a Transaction, the entity will not be automatically
    created in the Stark Bank API. The 'create' function sends the objects
    to the Stark Bank API and returns the list of created objects.
    ## Parameters (required):
    - amount [integer]: amount in cents to be transferred. ex: 1234 (= R$ 12.34)
    - description [string]: text to be displayed in the receiver and the sender statements (Min. 10 characters). ex: "funds redistribution"
    - external_id [string]: unique id, generated by user, to avoid duplicated transactions. ex: "transaction ABC 2020-03-30"
    - receiver_id [string]: unique id of the receiving workspace. ex: "5656565656565656"
    ## Parameters (optional):
    - tags [list of strings]: list of strings for reference when searching transactions (may be empty). ex: ["abc", "test"]
    ## Attributes (return-only):
    - sender_id [string]: unique id of the sending workspace. ex: "5656565656565656"
    - source [string]: locator of the entity that generated the transaction. ex: "charge/1827351876292", "transfer/92873912873/chargeback"
    - id [string]: unique id returned when Transaction is created. ex: "7656565656565656"
    - fee [integer]: fee charged when transaction is created. ex: 200 (= R$ 2.00)
    - balance [integer]: account balance after transaction was processed. ex: 100000000 (= R$ 1,000,000.00)
    - created [datetime.datetime]: creation datetime for the transaction. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, amount, description, external_id, receiver_id, sender_id=None, tags=None, id=None, fee=None, created=None, source=None, balance=None):
        Resource.__init__(self, id=id)

        self.amount = amount
        self.description = description
        self.tags = tags
        self.external_id = external_id
        self.receiver_id = receiver_id
        self.sender_id = sender_id
        self.fee = fee
        self.created = check_datetime(created)
        self.source = source
        self.balance = balance


_resource = {"class": Transaction, "name": "Transaction"}


def create(transactions, user=None):
    """
    Deprecated: Function deprecated since v2.31.0
    """
    """# Create Transactions
    Send a list of Transaction objects for creation in the Stark Bank API
    ## Parameters (required):
    - transactions [list of Transaction objects]: list of Transaction objects to be created in the API
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of Transaction objects with updated attributes
    """
    raise StarkError([{"code": "deprecated", "message": "Function deprecated since v2.31.0"}])


def get(id, user=None):
    """# Retrieve a specific Transaction
    Receive a single Transaction object previously created in the Stark Bank API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - Transaction object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def query(limit=None, after=None, before=None, tags=None, external_ids=None, ids=None, user=None):
    """# Retrieve Transactions
    Receive a generator of Transaction objects previously created in the Stark Bank API.
    Use this function instead of page if you want to stream Transactions without worrying about cursors and pagination.
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - external_ids [list of strings, default None]: list of external ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - generator of Transaction objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        tags=tags,
        external_ids=external_ids,
        ids=ids,
        user=user,
    )


def page(cursor=None, limit=None, after=None, before=None, tags=None, external_ids=None, ids=None, user=None):
    """# Retrieve paged Transactions
    Receive a list of up to 100 Transaction objects previously created in the Stark Bank API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. It must be an integer between 1 and 100. ex: 50
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - external_ids [list of strings, default None]: list of external ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of Transaction objects with updated attributes
    - cursor to retrieve the next page of Transaction objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        tags=tags,
        external_ids=external_ids,
        ids=ids,
        user=user,
    )



from re import search
from setuptools import setup, find_packages

try:
    with open('README.md', encoding="utf-8") as f:
        README = f.read()
except:
    with open('README.md') as f:
        README = f.read()

with open('starkbank/__init__.py') as f:
    version = search(r'version = \"(.*)\"', f.read()).group(1)

setup(
    name="starkbank",
    packages=find_packages(),
    include_package_data=True,
    description="SDK to facilitate Python integrations with Stark Bank",
    long_description=README,
    long_description_content_type="text/markdown",
    license="MIT License",
    url="https://github.com/starkbank/sdk-python",
    author="Stark Bank",
    author_email="developers@starkbank.com",
    keywords=["stark bank", "starkbank", "sdk", "open banking", "openbanking", "banking", "open", "stark"],
    version=version,
    install_requires=[
        "starkcore>=0.0.5",
    ],
)



from starkcore.utils.subresource import SubResource


class AllowedInstallment(SubResource):
    def __init__(self, total_amount, count):
        super().__init__()
        self.total_amount = total_amount
        self.count = count


_sub_resource = {"class": AllowedInstallment, "name": "AllowedInstallment"}




from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date
from ..utils import rest
from ..corporaterule import parse_rules
from .__permission import parse_permissions


class CorporateHolder(Resource):
    """# CorporateHolder object
    The CorporateHolder describes a card holder that may group several cards.
    When you initialize a CorporateHolder, the entity will not be automatically
    created in the Stark Bank API. The 'create' function sends the objects
    to the Stark Bank API and returns the created object.
    ## Parameters (required):
    - name [string]: cardholder name. ex: "Tony Stark"
    ## Parameters (optional):
    - center_id [string, default None]: target cost center ID. ex: "5656565656565656"
    - permissions [list of Permission object, default None]: list of Permission object representing access granted to an user for a particular cardholder.
    - rules [list of CorporateRule, default []]: [EXPANDABLE] list of holder spending rules
    - tags [list of strings, default []]: list of strings for tagging. ex: ["travel", "food"]
    ## Attributes (return-only):
    - id [string]: unique id returned when CorporateHolder is created. ex: "5656565656565656"
    - status [string]: current CorporateHolder status. ex: "active", "blocked", "canceled"
    - updated [datetime.datetime]: latest update datetime for the CorporateHolder. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - created [datetime.datetime]: creation datetime for the CorporateHolder. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, name, center_id=None, permissions=None, rules=None, tags=None, id=None, status=None, updated=None, created=None):
        Resource.__init__(self, id=id)

        self.name = name
        self.center_id = center_id
        self.permissions = parse_permissions(permissions)
        self.rules = parse_rules(rules)
        self.tags = tags
        self.status = status
        self.updated = check_datetime(updated)
        self.created = check_datetime(created)


_resource = {"class": CorporateHolder, "name": "CorporateHolder"}


def create(holders, expand=None, user=None):
    """# Create CorporateHolders
    Send a list of CorporateHolder objects for creation at the Stark Bank API
    ## Parameters (required):
    - holders [list of CorporateHolder objects]: list of CorporateHolder objects to be created in the API
    ## Parameters (optional):
    - expand [list of strings, default None]: fields to expand information. Options: ["rules"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of CorporateHolder objects with updated attributes
    """
    return rest.post_multi(resource=_resource, entities=holders, expand=expand, user=user)


def get(id, expand=None, user=None):
    """# Retrieve a specific CorporateHolder
    Receive a single CorporateHolder object previously created in the Stark Bank API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - expand [list of strings, default None]: fields to expand information. Options: ["rules"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - CorporateHolder object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, expand=expand, user=user)


def query(limit=None, after=None, before=None, ids=None, status=None, tags=None, expand=None, user=None):
    """# Retrieve CorporateHolders
    Receive a generator of CorporateHolder objects previously created in the Stark Bank API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - status [list of strings, default None]: filter for status of retrieved objects. ex: ["active", "blocked", "canceled"]
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - expand [string, default None]: fields to expand information. Options: ["rules"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - generator of CorporateHolder objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        ids=ids,
        status=status,
        tags=tags,
        expand=expand,
        user=user,
    )


def page(cursor=None, limit=None, after=None, before=None, ids=None, status=None, tags=None, expand=None, user=None):
    """# Retrieve CorporateHolders
    Receive a list of up to 100 CorporateHolder objects previously created in the Stark Bank API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. Max = 100. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - status [list of strings, default None]: filter for status of retrieved objects. ex: ["active", "blocked", "canceled"]
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - expand [string, default None]: fields to expand information. Options: ["rules"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of CorporateHolder objects with updated attributes
    - cursor to retrieve the next page of CorporateHolder objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        ids=ids,
        status=status,
        tags=tags,
        expand=expand,
        user=user,
    )


def update(id, center_id=None, permissions=None, status=None, name=None, rules=None, tags=None, user=None):
    """# Update CorporateHolder entity
    Update a CorporateHolder by passing its id.
    ## Parameters (required):
    - id [string]: CorporateHolder id. ex: '5656565656565656'
    ## Parameters (optional):
    - center_id [string, default None]: target cost center ID. ex: "5656565656565656"
    - permissions [list of Permission object, default None]: list of Permission object representing access granted to an user for a particular cardholder.
    - status [string, default None]: You may block the CorporateHolder by passing 'blocked' in the status
    - name [string, default None]: card holder name.
    - tags [list of strings, default None]: list of strings for tagging
    - rules [list of dictionaries, default None]: list of dictionaries with "amount": int, "currencyCode": string, "id": string, "interval": string, "name": string pairs
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - target CorporateHolder with updated attributes
    """
    payload = {
        "centerId": center_id,
        "permissions": permissions,
        "status": status,
        "name": name,
        "rules": rules,
        "tags": tags,
    }
    return rest.patch_id(resource=_resource, id=id, user=user, payload=payload)


def cancel(id, user=None):
    """# Cancel a CorporateHolder entity
    Cancel a CorporateHolder entity previously created in the Stark Bank API
    ## Parameters (required):
    - id [string]: CorporateHolder unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - canceled CorporateHolder object
    """
    return rest.delete_id(resource=_resource, id=id, user=user)



from starkcore.utils.api import from_api_json
from starkcore.utils.checks import check_datetime
from starkcore.utils.subresource import SubResource


class Permission(SubResource):
    """# corporateholder.Permission object
    Permission object represents access granted to an user for a particular cardholder
    ## Parameters (optional):
    - owner_id [string, default None]: owner unique id. ex: "5656565656565656"
    - owner_type [string, default None]: owner type. ex: "project"
    ## Attributes (return only):
    - owner_email [string]: email address of the owner. ex: "tony@starkbank.com
    - owner_name [string]: name of the owner. ex: "Tony Stark"
    - owner_picture_url [string]: Profile picture Url of the owner. ex: "https://storage.googleapis.com/api-ms-workspace-sbx.appspot.com/pictures/member/6227829385592832?20230404164942"
    - owner_status [string]: current owner status. ex: "active", "blocked", "canceled"
    - created [datetime.datetime]: creation datetime for the Permission. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, owner_email=None, owner_id=None, owner_name=None, owner_picture_url=None, owner_status=None,
                 owner_type=None, created=None):
        self.owner_email = owner_email
        self.owner_id = owner_id
        self.owner_name = owner_name
        self.owner_picture_url = owner_picture_url
        self.owner_status = owner_status
        self.owner_type = owner_type
        self.created = check_datetime(created)


_resource = {"class": Permission, "name": "Permission"}


def parse_permissions(permissions):
    parsed_permissions = []
    if permissions is None:
        return permissions
    for permission in permissions:
        if isinstance(permission, Permission):
            parsed_permissions.append(permission)
            continue
        parsed_permissions.append(from_api_json(_resource, permission))
    return parsed_permissions



from json import dumps
from ..utils import rest
from ..utils.parse import parse_and_verify
from starkcore.utils.api import api_json
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date


class CorporatePurchase(Resource):
    """# CorporatePurchase object
    Displays the CorporatePurchase objects created in your Workspace.
    ## Attributes (return-only):
    - id [string]: unique id returned when CorporatePurchase is created. ex: "5656565656565656"
    - holder_id [string]: card holder unique id. ex: "5656565656565656"
    - holder_name [string]: card holder name. ex: "Tony Stark"
    - center_id [string]: target cost center ID. ex: "5656565656565656"
    - card_id [string]: unique id returned when CorporateCard is created. ex: "5656565656565656"
    - card_ending [string]: last 4 digits of the card number. ex: "1234"
    - description [string]: purchase descriptions. ex: "my_description"
    - amount [integer]: CorporatePurchase value in cents. Minimum = 0. ex: 1234 (= R$ 12.34)
    - tax [integer]: IOF amount taxed for international purchases. ex: 1234 (= R$ 12.34)
    - issuer_amount [integer]: issuer amount. ex: 1234 (= R$ 12.34)
    - issuer_currency_code [string]: issuer currency code. ex: "USD"
    - issuer_currency_symbol [string]: issuer currency symbol. ex: "$"
    - merchant_amount [integer]: merchant amount. ex: 1234 (= R$ 12.34)
    - merchant_currency_code [string]: merchant currency code. ex: "USD"
    - merchant_currency_symbol [string]: merchant currency symbol. ex: "$"
    - merchant_category_code [string]: merchant category code. ex: "fastFoodRestaurants"
    - merchant_category_type [string]: merchant category type. ex: "health"
    - merchant_country_code [string]: merchant country code. ex: "USA"
    - merchant_name [string]: merchant name. ex: "Google Cloud Platform"
    - merchant_display_name [string]: merchant name. ex: "Google Cloud Platform"
    - merchant_display_url [string]: public merchant icon (png image). ex: "https://sandbox.api.starkbank.com/v2/corporate-icon/merchant/ifood.png"
    - merchant_fee [integer]: fee charged by the merchant to cover specific costs, such as ATM withdrawal logistics, etc. ex: 200 (= R$ 2.00)
    - method_code [string]: method code. Options: "chip", "token", "server", "manual", "magstripe" or "contactless"
    - tags [list of strings]: list of strings for tagging returned by the sub-issuer during the authorization. ex: ["travel", "food"]
    - corporate_transaction_ids [list of strings]: ledger transaction ids linked to this Purchase
    - status [string]: current CorporateCard status. Options: "approved", "canceled", "denied", "confirmed", "voided"
    - updated [datetime.datetime]: latest update datetime for the CorporatePurchase. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - created [datetime.datetime]: creation datetime for the CorporatePurchase. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, id=None, holder_id=None, holder_name=None, center_id=None, card_id=None, card_ending=None, 
                 description=None, amount=None, tax=None, issuer_amount=None, issuer_currency_code=None, issuer_currency_symbol=None, 
                 merchant_amount=None, merchant_currency_code=None, merchant_currency_symbol=None, merchant_category_code=None, 
                 merchant_category_type=None, merchant_country_code=None, merchant_name=None, merchant_display_name=None, 
                 merchant_display_url=None, merchant_fee=None, method_code=None, tags=None, corporate_transaction_ids=None, 
                 status=None, updated=None, created=None):
        Resource.__init__(self, id=id)

        self.id = id
        self.holder_id = holder_id
        self.holder_name = holder_name
        self.center_id = center_id
        self.card_id = card_id
        self.card_ending = card_ending
        self.description = description
        self.amount = amount
        self.tax = tax
        self.issuer_amount = issuer_amount
        self.issuer_currency_code = issuer_currency_code
        self.issuer_currency_symbol = issuer_currency_symbol
        self.merchant_amount = merchant_amount
        self.merchant_currency_code = merchant_currency_code
        self.merchant_currency_symbol = merchant_currency_symbol
        self.merchant_category_code = merchant_category_code
        self.merchant_category_type = merchant_category_type
        self.merchant_country_code = merchant_country_code
        self.merchant_name = merchant_name
        self.merchant_display_name = merchant_display_name
        self.merchant_display_url = merchant_display_url
        self.merchant_fee = merchant_fee
        self.method_code = method_code
        self.tags = tags
        self.corporate_transaction_ids = corporate_transaction_ids
        self.status = status
        self.updated = check_datetime(updated)
        self.created = check_datetime(created)


_resource = {"class": CorporatePurchase, "name": "CorporatePurchase"}


def get(id, user=None):
    """# Retrieve a specific CorporatePurchase
    Receive a single CorporatePurchase object previously created in the Stark Bank API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - CorporatePurchase object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def query(ids=None, limit=None, after=None, before=None, merchant_category_types=None, holder_ids=None, card_ids=None,
          status=None, user=None):
    """# Retrieve CorporatePurchases
    Receive a generator of CorporatePurchase objects previously created in the Stark Bank API
    ## Parameters (optional):
    - ids [list of strings, default None]: purchase IDs
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - merchant_category_types [list of strings, default None]: merchant category type. ex: "health"
    - holder_ids [list of strings, default None]: card holder IDs. ex: ["5656565656565656", "4545454545454545"]
    - card_ids [list of strings, default None]: card  IDs. ex: ["5656565656565656", "4545454545454545"]
    - status [list of strings, default None]: filter for status of retrieved objects. ex: ["approved", "canceled", "denied", "confirmed", "voided"],
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - generator of CorporatePurchase objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        ids=ids,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        merchant_category_types=merchant_category_types,
        holder_ids=holder_ids,
        card_ids=card_ids,
        status=status,
        user=user,
    )


def page(merchant_category_types=None, holder_ids=None, card_ids=None, status=None, after=None, before=None, ids=None,
        cursor=None, limit=None, user=None):
    """# Retrieve paged CorporatePurchase
    Receive a list of up to 100 CorporatePurchase objects previously created in the Stark Bank API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. Max = 100. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - merchant_category_types [list of strings, default None]: merchant category type. ex: "health"
    - holder_ids [list of strings, default None]: card holder IDs. ex: ["5656565656565656", "4545454545454545"]
    - card_ids [list of strings, default None]: card  IDs. ex: ["5656565656565656", "4545454545454545"]
    - status [list of strings, default None]: filter for status of retrieved objects. ex: ["approved", "canceled", "denied", "confirmed", "voided"]
    - ids [list of strings, default None]: purchase IDs
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of CorporatePurchase objects with updated attributes
    - cursor to retrieve the next page of CorporatePurchase objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        ids=ids,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        merchant_category_types=merchant_category_types,
        holder_ids=holder_ids,
        card_ids=card_ids,
        status=status,
        user=user,
    )


def parse(content, signature, user=None):
    """# Create a single verified CorporatePurchase authorization request from a content string
    Use this method to parse and verify the authenticity of the authorization request received at the informed endpoint.
    Authorization requests are posted to your registered endpoint whenever CorporatePurchases are received.
    They present CorporatePurchase data that must be analyzed and answered with approval or declination.
    If the provided digital signature does not check out with the starkbank public key, a stark.exception.InvalidSignatureException will be raised.
    If the authorization request is not answered within 2 seconds or is not answered with an HTTP status code 200 the CorporatePurchase will go through the pre-configured stand-in validation.
    ## Parameters (required):
    - content [string]: response content from request received at user endpoint (not parsed)
    - signature [string]: base-64 digital signature received at response header "Digital-Signature"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - Parsed CorporatePurchase object
    """
    return parse_and_verify(
        content=content,
        signature=signature,
        user=user,
        resource=_resource,
        key="",
    )


def response(status, amount=None, reason=None, tags=None):
    """# Helps you respond CorporatePurchase requests
    ## Parameters (required):
    - status [string]: sub-issuer response to the authorization. ex: "approved" or "denied"
    ## Parameters (conditionally required):
    - reason [string]: denial reason. Options: "other", "blocked", "lostCard", "stolenCard", "invalidPin", "invalidCard", "cardExpired", "issuerError", "concurrency", "standInDenial", "subIssuerError", "invalidPurpose", "invalidZipCode", "invalidWalletId", "inconsistentCard", "settlementFailed", "cardRuleMismatch", "invalidExpiration", "prepaidInstallment", "holderRuleMismatch", "insufficientBalance", "tooManyTransactions", "invalidSecurityCode", "invalidPaymentMethod", "confirmationDeadline", "withdrawalAmountLimit", "insufficientCardLimit", "insufficientHolderLimit"
    ## Parameters (optional):
    - amount [integer, default None]: amount in cents that was authorized. ex: 1234 (= R$ 12.34)
    - tags [list of strings, default None]: tags to filter retrieved object. ex: ["tony", "stark"]
    ## Return:
    - Dumped JSON string that must be returned to us on the CorporatePurchase request
    """
    params = {"authorization": {
        "status": status,
        "amount": amount,
        "reason": reason,
        "tags": tags,
    }}
    return dumps(api_json(params))



from ..utils import rest
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_datetime_or_date, check_date


class InvoicePullSubscription(Resource):
    """# InvoicePullSubscription object
    When you initialize an InvoicePullSubscription, the entity will not be automatically
    sent to the Stark Bank API. The 'create' function sends the objects
    to the Stark Bank API and returns the list of created objects.
    ## Parameters (required):
    - start [datetime.date or string]: subscription start date. ex: "2022-04-01"
    - interval [string]: subscription installment interval. Options: "week", "month", "quarter", "semester", "year"
    - pull_mode [string]: subscription pull mode. Options: "manual", "automatic". Automatic mode will create the Invoice Pull Requests automatically
    - pull_retry_limit [integer]: subscription pull retry limit. Options: 0, 3
    - type [string]: subscription type. Options: "push", "qrcode", "qrcodeAndPayment", "paymentAndOrQrcode"
    ## Parameters (conditionally required):
    - amount [integer, default 0]: subscription amount in cents. Required if an amount_min_limit is not informed. Minimum = 1 (R$ 0.01). ex: 100 (= R$ 1.00)
    - amount_min_limit [integer, 0 None]: subscription minimum amount in cents. Required if an amount is not informed. Minimum = 1 (R$ 0.01). ex: 100 (= R$ 1.00)
    ## Parameters (optional):
    - display_description [string, default None]: Invoice description to be shown to the payer. ex: "Subscription payment"
    - due [datetime.timedelta or integer, default None]: subscription invoice due offset. Available only for type "push". ex: timedelta(days=7)
    - external_id [string, default None]: string that must be unique among all your InvoicePullSubscriptions. Duplicated external_ids will cause failures. ex: "my-external-id"
    - reference_code [string, default None]: reference code for reconciliation. ex: "REF123456"
    - end [datetime.date or string, default None]: subscription end date. ex: "2023-04-01"
    - data [dictionary, default None]: additional data for the subscription based on type
    - name [string, default None]: subscription debtor name. ex: "Iron Bank S.A."
    - tax_id [string, default None]: subscription debtor tax ID (CPF or CNPJ) with or without formatting. ex: "01234567890" or "20.018.183/0001-80"
    - tags [list of strings, default []]: list of strings for tagging
    ## Attributes (return-only):
    - id [string]: unique id returned when InvoicePullSubscription is created. ex: "5656565656565656"
    - status [string]: current InvoicePullSubscription status. ex: "active", "canceled"
    - bacen_id [string]: unique authentication id at the Central Bank. ex: "RR2001818320250616dtsPkBVaBYs"
    - brcode [string]: Brcode string for the InvoicePullSubscription. ex: "00020101021126580014br.gov.bcb.pix0114+5599999999990210starkbank.com.br520400005303986540410000000000005802BR5913Stark Bank S.A.6009SAO PAULO62070503***6304D2B1"
    - created [datetime.datetime]: creation datetime for the InvoicePullSubscription. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - updated [datetime.datetime]: latest update datetime for the InvoicePullSubscription. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, start, interval, pull_mode, pull_retry_limit, type, amount=None, amount_min_limit=None,
                 display_description=None, due=None, external_id=None, reference_code=None, end=None, data=None,
                 name=None, tax_id=None, tags=None, id=None, status=None, bacen_id=None, brcode=None, created=None, updated=None):
        Resource.__init__(self, id=id)

        self.start = check_datetime_or_date(start)
        self.interval = interval
        self.pull_mode = pull_mode
        self.pull_retry_limit = pull_retry_limit
        self.type = type
        self.amount = amount
        self.amount_min_limit = amount_min_limit
        self.display_description = display_description
        self.due = None if due == "" else check_datetime_or_date(due)
        self.external_id = external_id
        self.reference_code = reference_code
        self.end = None if end == "" else check_datetime_or_date(end)
        self.data = data
        self.name = name
        self.tax_id = tax_id
        self.tags = tags
        self.status = status
        self.bacen_id = bacen_id
        self.brcode = brcode
        self.created = check_datetime(created)
        self.updated = check_datetime(updated)


_resource = {"class": InvoicePullSubscription, "name": "InvoicePullSubscription"}


def create(subscriptions, user=None):
    """# Create InvoicePullSubscriptions
    Send a list of InvoicePullSubscription objects for creation in the Stark Bank API
    ## Parameters (required):
    - subscriptions [list of InvoicePullSubscription objects]: list of InvoicePullSubscription objects to be created in the API
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of InvoicePullSubscription objects with updated attributes
    """
    return rest.post_multi(resource=_resource, entities=subscriptions, user=user)


def get(id, user=None):
    """# Retrieve a specific InvoicePullSubscription
    Receive a single InvoicePullSubscription object previously created in the Stark Bank API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - InvoicePullSubscription object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def query(limit=None, status=None, invoice_ids=None, external_ids=None, tags=None, 
          ids=None, after=None, before=None, user=None):
    """# Retrieve InvoicePullSubscriptions
    Receive a generator of InvoicePullSubscription objects previously created in the Stark Bank API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. ex: ["active", "canceled"]
    - invoice_ids [list of strings, default None]: list of Invoice ids linked to the subscriptions. ex: ["5656565656565656", "4545454545454545"]
    - external_ids [list of strings, default None]: list of external_ids to filter retrieved objects. ex: ["my-external-id-1", "my-external-id-2"]
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - generator of InvoicePullSubscription objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        invoice_ids=invoice_ids,
        external_ids=external_ids,
        tags=tags,
        ids=ids,
        user=user,
    )


def page(cursor=None, limit=None, status=None, invoice_ids=None, external_ids=None,
         tags=None, ids=None, after=None, before=None, user=None):
    """# Retrieve paged InvoicePullSubscriptions
    Receive a list of up to 100 InvoicePullSubscription objects previously created in the Stark Bank API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. It must be an integer between 1 and 100. ex: 50
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. ex: ["active", "canceled"]
    - invoice_ids [list of strings, default None]: list of Invoice ids linked to the subscriptions. ex: ["5656565656565656", "4545454545454545"]
    - external_ids [list of strings, default None]: list of external_ids to filter retrieved objects. ex: ["my-external-id-1", "my-external-id-2"]
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of InvoicePullSubscription objects with updated attributes
    - cursor to retrieve the next page of InvoicePullSubscription objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        invoice_ids=invoice_ids,
        external_ids=external_ids,
        tags=tags,
        ids=ids,
        user=user,
    )


def cancel(id, user=None):
    """# Cancel an InvoicePullSubscription entity
    Cancel an InvoicePullSubscription entity previously created in the Stark Bank API
    ## Parameters (required):
    - id [string]: InvoicePullSubscription unique id. ex: '5656565656565656'
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - canceled InvoicePullSubscription object
    """
    return rest.delete_id(resource=_resource, id=id, user=user)



from ..utils import rest
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_date
from starkcore.utils.checks import check_datetime


class DictKey(Resource):

    """# DictKey object
    DictKey represents a PIX key registered in Bacen's DICT system.
    ## Parameters (optional):
    - id [string]: DictKey object unique id. ex: "tony@starkbank.com", "722.461.430-04", "20.018.183/0001-80", "+5511988887777", "b6295ee1-f054-47d1-9e90-ee57b74f60d9"
    ## Attributes (return-only):
    - type [string]: DICT key type. ex: "email", "cpf", "cnpj", "phone" or "evp"
    - name [string]: key owner full name. ex: "Tony Stark"
    - tax_id [string]: key owner tax ID (CNPJ or masked CPF). ex: "***.345.678-**" or "20.018.183/0001-80"
    - owner_type [string]: DICT key owner type. ex "naturalPerson" or "legalPerson"
    - bank_name [string]: bank name associated with the DICT key. ex: "Stark Bank"
    - ispb [string]: bank ISPB associated with the DICT key. ex: "20018183"
    - branch_code [string]: encrypted bank account branch code associated with the DICT key. ex: "ZW5jcnlwdGVkLWJyYW5jaC1jb2Rl"
    - account_number [string]: encrypted bank account number associated with the DICT key. ex: "ZW5jcnlwdGVkLWFjY291bnQtbnVtYmVy"
    - account_type [string]: bank account type associated with the DICT key. ex: "checking", "savings", "salary" or "payment"
    - status [string]: current DICT key status. ex: "created", "registered", "canceled" or "failed"
    """
    
    def __init__(self, id=None, type=None, name=None, tax_id=None, owner_type=None, bank_name=None, 
                   ispb=None, branch_code=None, account_number=None, account_type=None, status=None):
        Resource.__init__(self, id=id)

        self.type = type
        self.name = name
        self.tax_id = tax_id
        self.owner_type = owner_type
        self.bank_name = bank_name
        self.ispb = ispb
        self.branch_code = branch_code
        self.account_number = account_number
        self.account_type = account_type
        self.status = status


_resource = {"class": DictKey, "name": "DictKey"}


def get(id, user=None):
    """# Retrieve a specific DictKey
    Receive a single DictKey object by its id
    ## Parameters (required):
    - id [string]: DictKey object unique id and PIX key itself. ex: "tony@starkbank.com", "722.461.430-04", "20.018.183/0001-80", "+5511988887777", "b6295ee1-f054-47d1-9e90-ee57b74f60d9"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - DictKey object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def query(limit=None, type=None, after=None, before=None, ids=None, status=None, user=None):
    """# Retrieve DictKeys
    Receive a generator of DictKey objects associated with your Stark Bank Workspace
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - type [string, default None]: DictKey type. ex: "cpf", "cnpj", "phone", "email" or "evp"
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - status [string, default None]: filter for status of retrieved objects. ex: "success"
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - generator of DictKey objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        type=type,
        after=check_date(after),
        before=check_date(before),
        ids=ids,
        status=status,
        user=user,
    )


def page(cursor=None, limit=None, type=None, after=None, before=None, ids=None, status=None, user=None):
    """# Retrieve paged DictKeys
    Receive a list of up to 100 DictKey objects previously created in the Stark Bank API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. It must be an integer between 1 and 100. ex: 50
    - type [string, default None]: DictKey type. ex: "cpf", "cnpj", "phone", "email" or "evp"
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - status [string, default None]: filter for status of retrieved objects. ex: "success"
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of DictKey objects with updated attributes
    - cursor to retrieve the next page of DictKey objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        type=type,
        after=check_date(after),
        before=check_date(before),
        ids=ids,
        status=status,
        user=user,
    )



from ..utils import rest
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date


class Split(Resource):

    """# Split object
    When you initialize a Split, the entity will not be automatically
    created in the Stark Bank API. The 'create' function sends the objects
    to the Stark Bank API and returns the list of created objects.
    ## Parameters (required):
    - amount [int]: value to send to receivers. ex: 1000 (= R$ 10.00)
    - receiver_id [string]: split receiver unique id. ex: "5656565656565656"
    ## Attributes (return-only):
    - id [string]: unique id returned when split is created. ex: "5656565656565656"
    - source [string]: source receivable which generated this split object. ex: "5656565656565656"
    - external_id [string]: unique id, generated by the system, to avoid duplicated splits. ex: "invoice/1234/receiver/5678"
    - tags [list of strings, default None]: list of strings for tagging
    - scheduled [datetime.date, datetime.datetime or string, default now]: payment scheduled date or datetime. ex: datetime.datetime(2020, 3, 10, 15, 17, 3)
    - status [string]: current payment status. ex: "success" or "failed"
    - created [datetime.datetime]: creation datetime for the payment. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - updated [datetime.datetime]: update datetime for the payment. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, amount, receiver_id, id=None, source=None, external_id=None,
                 tags=None, scheduled=None, status=None, created=None, updated=None):
        Resource.__init__(self, id=id)

        self.amount = amount
        self.receiver_id = receiver_id
        self.source = source
        self.external_id = external_id
        self.tags = tags
        self.scheduled = check_datetime(scheduled)
        self.status = status
        self.created = check_datetime(created)
        self.updated = check_datetime(updated)


_resource = {"class": Split, "name": "Split"}


def get(id, user=None):
    """# Retrieve a specific Split
    Receive a single Split object previously created by the Stark Bank API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - Split object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def query(limit=None, after=None, before=None, tags=None, ids=None, receiver_ids=None, status=None, user=None):
    """# Retrieve Splits
    Receive a generator of Split objects previously created in the Stark Bank API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - receiver_ids [list of strings, default None]: list of receiver ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - status [string, default None]: filter for status of retrieved objects. ex: "success"
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - generator of Split objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        tags=tags,
        ids=ids,
        receiver_ids=receiver_ids,
        status=status,
        user=user,
    )


def page(cursor=None, after=None, before=None, tags=None, ids=None, receiver_ids=None, status=None, limit=None, user=None):
    """# Retrieve paged Splits
    Receive a list of up to 100 Split objects previously created in the Stark Bank API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. It must be an integer between 1 and 100. ex: 50
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - receiver_ids [list of strings, default None]: list of receiver ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - status [string, default None]: filter for status of retrieved objects. ex: "success"
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of Split objects with updated attributes
    - cursor to retrieve the next page of Split objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        tags=tags,
        ids=ids,
        receiver_ids=receiver_ids,
        status=status,
        user=user,
    )



from ..utils import rest
from starkcore.utils.resource import Resource
from starkcore.utils.api import from_api_json
from starkcore.utils.checks import check_date, check_datetime, check_timedelta, check_datetime_or_date
from .__payment import _sub_resource as _payment_sub_resource
from .rule.__rule import Rule
from .rule.__rule import _sub_resource as _rule_resource
from ..split.__split import _resource as _split_resource, Split


class Invoice(Resource):
    """# Invoice object
    When you initialize an Invoice, the entity will not be automatically
    sent to the Stark Bank API. The 'create' function sends the objects
    to the Stark Bank API and returns the list of created objects.
    To create scheduled Invoices, which will display the discount, interest, etc. on the final users banking interface,
    use dates instead of datetimes on the "due" and "discounts" fields.
    ## Parameters (required):
    - amount [integer]: Invoice value in cents. Minimum = 0 (any value will be accepted). ex: 1234 (= R$ 12.34)
    - tax_id [string]: payer tax ID (CPF or CNPJ) with or without formatting. ex: "01234567890" or "20.018.183/0001-80"
    - name [string]: payer name. ex: "Iron Bank S.A."
    ## Parameters (optional):
    - due [datetime.datetime or datetime.date or string, default now + 2 days]: Invoice due date in UTC ISO format. ex: "2020-10-28T17:59:26.249976+00:00" for immediate invoices and "2020-10-28" for scheduled invoices
    - expiration [integer or datetime.timedelta, default 5097600 (59 days)]: time interval in seconds between due date and expiration date. ex 123456789
    - fine [float, default 2.0]: Invoice fine for overdue payment in %. ex: 2.5
    - interest [float, default 1.0]: Invoice monthly interest for overdue payment in %. ex: 5.2
    - discounts [list of dictionaries, default []]: list of dictionaries with "percentage":float and "due":datetime.datetime or string pairs
    - rules [list of Invoice.Rules, default []]: list of Invoice.Rule objects for modifying invoice behavior. ex: [Invoice.Rule(key="allowedTaxIds", value=[ "012.345.678-90", "45.059.493/0001-73" ])]
    - splits [list of Split.Splits, default []]: list of Split.Splits objects to indicate payment receivers. ex: [Invoice.Split(amount=141, receiverId="5706627130851328")]
    - tags [list of strings, default []]: list of strings for tagging
    - descriptions [list of dictionaries, default []]: list of dictionaries with "key":string and (optional) "value":string pairs
    ## Attributes (return-only):
    - pdf [string]: public Invoice PDF URL. ex: "https://invoice.starkbank.com/pdf/d454fa4e524441c1b0c1a729457ed9d8"
    - link [string]: public Invoice webpage URL. ex: "https://my-workspace.sandbox.starkbank.com/invoicelink/d454fa4e524441c1b0c1a729457ed9d8"
    - nominal_amount [integer]: Invoice emission value in cents (will change if invoice is updated, but not if it's paid). ex: 400000
    - fine_amount [integer]: Invoice fine value calculated over nominal_amount. ex: 20000
    - interest_amount [integer]: Invoice interest value calculated over nominal_amount. ex: 10000
    - discount_amount [integer]: Invoice discount value calculated over nominal_amount. ex: 3000
    - id [string]: unique id returned when Invoice is created. ex: "5656565656565656"
    - brcode [string]: BR Code for the Invoice payment. ex: "00020101021226800014br.gov.bcb.pix2558invoice.starkbank.com/f5333103-3279-4db2-8389-5efe335ba93d5204000053039865802BR5913Arya Stark6009Sao Paulo6220051656565656565656566304A9A0"
    - status [string]: current Invoice status. ex: "registered" or "paid"
    - fee [integer]: fee charged by this Invoice. ex: 200 (= R$ 2.00)
    - transaction_ids [list of strings]: ledger transaction ids linked to this Invoice (if there are more than one, all but the first are reversals or failed reversal chargebacks). ex: ["19827356981273"]
    - created [datetime.datetime]: creation datetime for the Invoice. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - updated [datetime.datetime]: latest update datetime for the Invoice. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, amount, tax_id, name, due=None, expiration=None, fine=None, interest=None, discounts=None,
                 rules=None, tags=None, descriptions=None, pdf=None, link=None, nominal_amount=None, fine_amount=None,
                 interest_amount=None, discount_amount=None, id=None, brcode=None, status=None, fee=None, splits=None,
                 transaction_ids=None, created=None, updated=None):
        Resource.__init__(self, id=id)

        self.amount = amount
        self.nominal_amount = nominal_amount
        self.fine_amount = fine_amount
        self.interest_amount = interest_amount
        self.discount_amount = discount_amount
        self.due = check_datetime_or_date(due)
        self.tax_id = tax_id
        self.name = name
        self.expiration = check_timedelta(expiration)
        self.fine = fine
        self.interest = interest
        self.discounts = discounts
        self.rules = _parse_rules(rules)
        self.splits = _parse_splits(splits)
        self.tags = tags
        self.pdf = pdf
        self.link = link
        self.descriptions = descriptions
        self.brcode = brcode
        self.status = status
        self.fee = fee
        self.transaction_ids = transaction_ids
        self.created = check_datetime(created)
        self.updated = check_datetime(updated)


_resource = {"class": Invoice, "name": "Invoice"}


def _parse_rules(rules):
    if rules is None:
        return None
    parsed_rules = []
    for rule in rules:
        if isinstance(rule, Rule):
            parsed_rules.append(rule)
            continue
        parsed_rules.append(from_api_json(_rule_resource, rule))
    return parsed_rules


def _parse_splits(splits):
    if splits is None:
        return None
    parsed_splits = []
    for split in splits:
        if isinstance(split, Split):
            parsed_splits.append(split)
            continue
        parsed_splits.append(from_api_json(_split_resource, split))
    return parsed_splits


def create(invoices, user=None):
    """# Create Invoices
    Send a list of Invoice objects for creation in the Stark Bank API
    ## Parameters (required):
    - invoices [list of Invoice objects]: list of Invoice objects to be created in the API
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of Invoice objects with updated attributes
    """
    return rest.post_multi(resource=_resource, entities=invoices, user=user)


def get(id, user=None):
    """# Retrieve a specific Invoice
    Receive a single Invoice object previously created in the Stark Bank API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - Invoice object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def query(limit=None, status=None, tags=None, ids=None, after=None, before=None, user=None):
    """# Retrieve Invoices
    Receive a generator of Invoice objects previously created in the Stark Bank API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - status [string, default None]: filter for status of retrieved objects. ex: "paid" or "registered"
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - generator of Invoice objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        tags=tags,
        ids=ids,
        user=user,
    )


def page(cursor=None, limit=None, status=None, tags=None, ids=None, after=None, before=None, user=None):
    """# Retrieve paged Invoices
    Receive a list of up to 100 Invoice objects previously created in the Stark Bank API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. It must be an integer between 1 and 100. ex: 50
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - status [string, default None]: filter for status of retrieved objects. ex: "paid" or "registered"
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - list of Invoice objects with updated attributes
    - cursor to retrieve the next page of Invoice objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        tags=tags,
        ids=ids,
        user=user,
    )


def update(id, status=None, amount=None, due=None, expiration=None, user=None):
    """# Update Invoice entity
    Update an Invoice by passing id, if it hasn't been paid yet.
    ## Parameters (required):
    - id [string]: Invoice id. ex: '5656565656565656'
    ## Parameters (optional):
    - status [string]: You may cancel the invoice by passing 'canceled' in the status
    - amount [string]: Nominal amount charged by the invoice. ex: 100 (R$1.00)
    - due [datetime.datetime or string, default now + 2 days]: Invoice due date in UTC ISO format. ex: "2020-10-28T17:59:26.249976+00:00"
    - expiration [integer or datetime.timedelta, default None]: time interval in seconds between the due date and the expiration date. ex 123456789
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - target Invoice with updated attributes
    """
    payload = {
        "status": status,
        "amount": amount,
        "due": check_datetime(due),
        "expiration": check_timedelta(expiration),
    }
    return rest.patch_id(resource=_resource, id=id, user=user, payload=payload)


def qrcode(id, size=7, user=None):
    """# Retrieve a specific Invoice QR Code png
    Receive a single Invoice QR Code in png format generated in the Stark Bank API by the invoice ID.
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - size [integer, default 7]: number of pixels in each "box" of the QR code. Minimum = 1, maximum = 50. ex: 12
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - Invoice png blob
    """
    return rest.get_content(resource=_resource, id=id, size=size, user=user, sub_resource_name="qrcode")


def pdf(id, user=None):
    """# Retrieve a specific Invoice pdf file
    Receive a single Invoice pdf receipt file generated in the Stark Bank API by its id.
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - Invoice pdf file
    """
    return rest.get_content(resource=_resource, id=id, user=user, sub_resource_name="pdf")


def payment(id, user=None):
    """# Retrieve a specific Invoice payment information
    Receive the Invoice.Payment sub-resource associated with a paid Invoice.
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkbank.user was set before function call
    ## Return:
    - Invoice.Payment sub-resource
    """
    return rest.get_sub_resource(resource=_resource, id=id, user=user, sub_resource=_payment_sub_resource)
